@@ -6,7 +6,7 @@ import sys
 import tensorflow as tf
 
 from hicml.DataStructures import LockedList
-from concurrent.futures import ThreadPoolExecutor
+from concurrent.futures import ThreadPoolExecutor,as_completed
 
 print("hic-straw", hs.__version__)
 FEATURE_KEY = 'feat_resolution'
@@ -52,7 +52,7 @@ def make_region(coordinates, width: int, resolution: int):
 class Oracle:
     def __init__(self, loop_list_path: str,
                  hic_file: str, models_dir: str, norm: str,
-                 stem:str, num_straw_workers:int, width: int = 56, chr_list: list = None):
+                 stem:str, num_straw_workers:int, output_dir:str, width: int = 56, chr_list: list = None):
         self.__loopList = self.load_loops(loop_list_path)
         self.__hic = hs.HiCFile(hic_file)
         self.__norm = norm
@@ -61,7 +61,8 @@ class Oracle:
         self.__modelsDir = models_dir
         self.__resolutions = sorted(find_resolutions_from_list(self.__loopList))
         self.__stem = stem
-        self.__numWorkers = num_straw_workers,
+        self.__numWorkers = num_straw_workers
+        self.__outputDir = output_dir
         self.__resolution_sub_lists = make_resolution_sub_lists(self.__loopList)
         self.__chr = self.make_chr_list(chr_list)
         self.__model = tf.keras.models.load_model(self.__modelsDir)
@@ -96,7 +97,7 @@ class Oracle:
 
     def makePredictions(self,chr):
         scores=[]
-        df_tmp = self.loopList[self.loopList['chr1']==chr]
+        df_tmp = self.__loopList[self.__loopList['chr1']==chr]
         df_tmp = df_tmp.reset_index(drop=True)
         if df_tmp.shape[0]==0:
             return f'{chr} does not have any loops.'
@@ -106,7 +107,7 @@ class Oracle:
           print (f'ORACLE is running at {reso} on {chr}', flush=True)
           mzd = self.__hic.getMatrixZoomData(chr, chr, "observed", self.__norm , "BP", reso)
           for i, row in df_tmp.iterrows():   
-            x1,x2,y1,y2 = self.makeRegion((row['chr1'],row['x1'],row['x2'],row['y1'],row['y2']),self.__width-1,reso)
+            x1,x2,y1,y2 = make_region((row['chr1'],row['x1'],row['x2'],row['y1'],row['y2']),self.__width-1,reso)
             mat = mzd.getRecordsAsMatrix(x1,x2,y1,y2)
             batch[i,:self.__width-1,:self.__width-1,0] = self.preprocess(mat)
           prediction = self.__model.predict(batch)
@@ -156,12 +157,12 @@ class Oracle:
     def run(self):
         results = []
         with ThreadPoolExecutor(max_workers = self.__numWorkers) as executor:
-          results = executor.map(self.makePredictions, self.__chr)
-          for r in results:
-            print (r)
+          results = [executor.submit(self.makePredictions, x) for x in self.__chr]
+          for r in as_completed(results):
+            print (r.result())
           print('All Oracle jobs submitted', flush=True)
           df = pd.concat(self.__oracleList,ignore_index=True).fillna('.')
-          df.to_csv(self.__stem+'_ORACLE_merged.bedpe', sep = "\t", index=False)
+          df.to_csv(self.__outputDir+'/'+self.__stem+'_ORACLE_merged.bedpe', sep = "\t", index=False)
 
 
     def run_single_thread(self):
@@ -183,12 +184,13 @@ if __name__ == "__main__":
     NORMALIZATION_TYPE = sys.argv[4]
     STEM = sys.argv[5]
     NUM_WORKERS = sys.argv[6]
-    if len(sys.argv) < 8:
-        CHR_LIST = ['All']
+    OUTPUT_DIR = sys.argv[7]
+    if len(sys.argv) < 9:
+        CHR_LIST = None
     else:
-        CHR_LIST = sys.argv[7].split(',')
+        CHR_LIST = sys.argv[8].split(',')
 
-    oracle = Oracle(BEDPE_DIR, FILEPATH, MODEL_DIR, NORMALIZATION_TYPE, STEM, NUM_WORKERS, chr_list=CHR_LIST)
+    oracle = Oracle(BEDPE_DIR, FILEPATH, MODEL_DIR, NORMALIZATION_TYPE, STEM, NUM_WORKERS, OUTPUT_DIR, chr_list=CHR_LIST)
     # passing, failing = oracle.run()
     oracle.run()
 
